/*************************************************************************
** funct-2.4      (command interpreter - funct)                          **
** main.c : main file                                                    **
**                                                                       **
** Copyright (C) 2000  Jean-Marc Drezet                                  **
**                                                                       **
**  This library is free software; you can redistribute it and/or        **
**  modify it under the terms of the GNU Library General Public          **
**  License as published by the Free Software Foundation; either         **
**  version 2 of the License, or (at your option) any later version.     **
**                                                                       **
**  This library is distributed in the hope that it will be useful,      **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    **
**  Library General Public License for more details.                     **
**                                                                       **
**  You should have received a copy of the GNU Library General Public    **
**  License along with this library; if not, write to the Free           **
**  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   **
**                                                                       **
** Please mail any bug reports/fixes/enhancements to me at:              **
**      drezet@math.jussieu.fr                                           **
** or                                                                    **
**      Jean-Marc Drezet                                                 **
**      Institut de Mathematiques                                        **
**      UMR 7586 du CNRS                                                 **
**      173, rue du Chevaleret                                           **
**      75013 Paris                                                      **
**      France                                                           **
**                                                                       **
 *************************************************************************/

#include "interp.h"
#include "funct_graph.h"

int              ind0k_gr;

#ifdef __GGILIB_X11
#include <ggi/ggi.h>
ggi_mode        s;
ggi_color       white;
ggi_color       black;
int             _nb_ggi_graph,
                _nb_ggi_color,
                ggi_line_width;
ggi_visual_t    ggi_vis;
ggi_color      *ggi_col;
ggi_pixel       color_ggi,
                color_white,
                color_black;
#endif
#ifdef __ALLEGROLIB_X11
#include <allegro.h>
int             _nb_allegro_color;
int            *allegro_col;
#endif

int   _XRANGE_F        = 1;
int   _XRANGE_D        = 2;
int   _RFUNC_F         = 3;
int   _RFUNC_D         = 4;
int   _CFUNC_F         = 5;
int   _CFUNC_D         = 6;
int   _FOUR_TR         = 7;
int   _BESS_PAR        = 8;
int   _FUNC_MESS       = 0;
int   _GRAPH_MESS      = 26;
int   _FUNC_GRAPH_MESS = 31;
int   _GRAPH_PS        = 9;
int   _GRAPH_COL       = 10;
int   _GRAPH_FRAM      = 11;
int   _GRAPH_X,
      _GRAPH_Y;
void  init_prog_graph(flow_data *);

pfi *proc[] =
{
    proc_func,
    proc_graph,
    proc_func_graph,
};

FUNCTION *Funcs;
FUNCTIONC *Funcs_C;
FUNCTIONGen *Funcs_Gen[] = {
};

extern int int_nb;
extern char int_txt[];



/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
int
main(int argc, char *argv[])
{
#ifdef _HAVE_THREADS
    pthread_mutex_init(&mutex_g2, NULL);
#endif
    Funcs = Funcs_func;
    Funcs_C = Funcs_interp_C;
    _NBFONC = _NBFONC_FUNC;
    _NBFONC_C = _NBFONC0_C;
    User_Init_File = ".funct";
    prog_c(argc, argv, NULL, int_txt, int_nb, NULL, 0);
    return 0;
}
#ifdef __ALLEGROLIB_X11
END_OF_MAIN();
#endif
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void
init_prog(flow_data *flow_interp)
{
    int             i;

    for (i = 0; i < 20; i++)
        _GRAPH_xx[i] = 0.;
    init_prog_graph(flow_interp);
    _GRAPH_X = S_convert_int("x_win", flow_interp);
    _GRAPH_Y = S_convert_int("y_win", flow_interp);
    ind0k_gr = 0;
#ifdef __GGILIB_X11
    white.r = 65500;
    white.g = 65500;
    white.b = 65500;
    black.r = 0;
    black.g = 0;
    black.b = 0;
    _nb_ggi_graph = -1;
    _nb_ggi_color = -1;
    ggi_col = (ggi_color *) malloc((size_t) S_convert_int("nbcol", flow_interp)
       * sizeof(ggi_color));
#endif
#ifdef __ALLEGROLIB_X11
    _nb_allegro_color = 0;
    allegro_col = (int *) malloc((size_t) S_convert_int("nbcol", flow_interp)
        * sizeof(int));
#endif
    _GRAPH_xx[8] = 1.;
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
     Function called by 'init_prog()'. Sets the default size of the graphics
---------------------------------------------------------------------------*/
void
init_prog_graph(flow_data *flow_interp)
{
    funct_clip     *f_clip;

    f_clip = (funct_clip *) flow_interp->extra[2];
    f_clip->xinf_clip = 0.15;
    f_clip->xsup_clip = 0.9;
    f_clip->yinf_clip = 0.1;
    f_clip->ysup_clip = 0.9;
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void
init_thread_param(flow_data *flow_interp)
{
    funct_clip     *f_clip;

    f_clip = (funct_clip *) malloc((size_t) sizeof(funct_clip));
    flow_interp->extra[2] = (char *) f_clip;
    init_prog_graph(flow_interp);
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void init_expr_GEN(flow_data *flow_interp)
{
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void
clean_thread_param(flow_data *flow_interp)
{
    funct_clip     *f_clip;

    f_clip = (funct_clip *) flow_interp->extra[2];
    free(f_clip);
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void
exit_prog()
{
    if (ind0k_gr != 0)
#ifdef __GGILIB_X11
        ggiExit();
#endif
#ifdef __ALLEGROLIB_X11
        allegro_exit();
#endif
    exit(0);
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void
dest_prop(int typ, int i0, flow_data *flow_interp)
{
    float          *xx;

    if (typ == _GRAPH_PS - 1) {
        xx = (float *) Obj[typ][i0].adresse;
        _FUNCT_CLOSE(typ + 1, (int) xx[2]);
    }
    dest_prop_func(typ, i0, flow_interp);
}
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
int
wait_cmd(int argc, char *argv[])
{
    getchar();
    return 0;
}
/*-------------------------------------------------------------------------*/



